==> em_celery-0.1.31-py2.py3-none-any.whl: em_celery/tools/ebay_products_update_seeder.py <==
# -*- coding: utf-8 -*-

import os
import sys
import datetime
import time
import json
try:
  from urllib.parse import urlparse
  from urllib.parse import parse_qs
except ImportError:
  from urlparse import urlparse
  from urlparse import parse_qs

import click
import redis
import dateparser

from em_tasks.store.store_util import StoreUtil
from em_tasks.spree.product_util import ProductUtil

from em_celery import logger, init_db, get_product_service


@click.command('Send Shopify ebay products crawl seed urls')
@click.option('-s', '--store_code', type=str, required=True, help='Store code.')
@click.option('-m', '--marketplace', type=str, default='us', help='Seeds add rate in seconds.')
@click.option('-u', '--redis_uri', required=True, type=str, help='Redis URI.')
@click.option('-q', '--qps', type=float, default=20, help='Seeds add rate in seconds.')
@click.option('-t', '--ttl', type=int, default=3, help='Product information alive days, default is 3.')
@click.option('-k', '--start_urls_key', required=True, type=str, default='ebay_product:start_urls', help='Start URLs key.')
def feed_seeds(store_code, redis_uri, qps=20, start_urls_key='ebay_product:start_urls', ttl=3, marketplace='us'):
  init_db()

  data_source = None
  store = StoreUtil.get_store_by_code(store_code)
  platform = store.platfrom
  if platform == 'Shopify':
    data_source = StoreUtil
  elif platform == 'Spree':
    credential = json.loads(store.api_credential)
    data_source = ProductUtil(credential['endpoint'], credential['api_key'], credential.get('api_version', 'v1'))
  else:
    logger.error('[UnsupportedPlatform] Only "Shopify" and "Spree" are supported, %s given', platform)
    return

  if not data_source:
    return

  feeder = EbayProductsFetchSeedFeeder(data_source, marketplace, redis_uri, start_urls_key, qps, ttl)
  feeder.run()


class EbayProductsFetchSeedFeeder():
  def __init__(self, data_source, marketplace, redis_uri, start_urls_key, qps, ttl=3):
    self.data_source = data_source
    self.marketplace = marketplace.upper()
    self.r = redis.Redis.from_url(redis_uri)
    self.start_urls_key = start_urls_key
    self.qps = qps
    self.last_send_time = None
    self.product_service = get_product_service()
    self.indice_name = 'ebay_{}_products'.format(marketplace.lower())
    today = datetime.date.today()
    self.expire_date = today - datetime.timedelta(days=ttl) 

  def run(self):
    product_ids = []
    batch_size = 500
    try:
      for prod in self.data_source.get_ebay_products(self.marketplace):
        product_ids.append(prod['source_product_id'])
        if len(product_ids) < batch_size:
          continue

        self.process_products(product_ids)
        product_ids = []
    except (KeyboardInterrupt, SystemExit):
      print('Exiting, please wait...')
    finally:
      if product_ids:
        self.process_products(product_ids)
        product_ids = []

  def process_products(self, product_ids):
    pids = dict()
    for pid in product_ids:
      pids[pid] = 'https://www.ebay.com/itm/{}'.format(pid)

    products = False
    max_retries = 3
    while max_retries > 0:
      try:
        products = self.product_service.search_products(self.indice_name, product_ids)
        if not isinstance(products, dict):
          logger.info('[ProductService] Temporary unavailable! Wait for 10 seconds to retry.')
          max_retries -= 1
          time.sleep(10)
          continue

        break
      except Exception as e:
        logger.exception(e)
        max_retries -= 1
        time.sleep(3)

    if not products:
      return

    for pid, product in products.items():
      # existence = product.get('existence', True)
      # if not existence:
      #   continue

      update_date_s = product.get('date', product.get('time', None))
      if not update_date_s:
        continue

      try:
        update_date = dateparser.parse(update_date_s).date()
        # Not expired, no need to update
        if update_date > self.expire_date:
          pids.pop(pid, None)
      except:
        continue

    urls = list(pids.values())
    for product_url in urls:
      if self.last_send_time:
        wait_time = 1 / self.qps - (time.time() - self.last_send_time)
        if wait_time > 0:
          print("Waiting %.3fs to send next message" % wait_time)
          time.sleep(wait_time)

      self.last_send_time = time.time()

      self.r.lpush(self.start_urls_key, json.dumps({'url': product_url}))
      logger.debug('Added %s', product_url)


if __name__ == "__main__":
  feed_seeds()

==> em_celery-0.1.36-py2.py3-none-any.whl: em_celery/tools/ebay_products_update_seeder.py <==
# -*- coding: utf-8 -*-

import os
import sys
import datetime
import time
import json
try:
  from urllib.parse import urlparse
  from urllib.parse import parse_qs
except ImportError:
  from urlparse import urlparse
  from urlparse import parse_qs

import click
import redis
import dateparser

from em_tasks.store.store_util import StoreUtil
from em_tasks.spree.product_util import ProductUtil

from em_celery import logger, init_db, get_product_service


@click.command('Send Shopify ebay products crawl seed urls')
@click.option('-s', '--store_code', type=str, required=True, help='Store code.')
@click.option('-m', '--marketplace', type=str, default='us', help='Seeds add rate in seconds.')
@click.option('-u', '--redis_uri', required=True, type=str, help='Redis URI.')
@click.option('-q', '--qps', type=float, default=20, help='Seeds add rate in seconds.')
@click.option('-t', '--ttl', type=int, default=3, help='Product information alive days, default is 3.')
@click.option('-k', '--start_urls_key', required=True, type=str, default='ebay_product:start_urls', help='Start URLs key.')
def feed_seeds(store_code, redis_uri, qps=20, start_urls_key='ebay_product:start_urls', ttl=3, marketplace='us'):
  init_db()

  data_source = None
  store = StoreUtil.get_store_by_code(store_code)
  platform = store.platform
  if platform == 'Shopify':
    data_source = StoreUtil
  elif platform == 'Spree':
    credential = json.loads(store.api_credential)
    data_source = ProductUtil(credential['endpoint'], credential['api_key'], credential.get('api_version', 'v1'))
  else:
    logger.error('[UnsupportedPlatform] Only "Shopify" and "Spree" are supported, %s given', platform)
    return

  if not data_source:
    return

  feeder = EbayProductsFetchSeedFeeder(data_source, marketplace, redis_uri, start_urls_key, qps, ttl)
  feeder.run()


class EbayProductsFetchSeedFeeder():
  def __init__(self, data_source, marketplace, redis_uri, start_urls_key, qps, ttl=3):
    self.data_source = data_source
    self.marketplace = marketplace.upper()
    self.r = redis.Redis.from_url(redis_uri)
    self.start_urls_key = start_urls_key
    self.qps = qps
    self.last_send_time = None
    self.product_service = get_product_service()
    self.indice_name = 'ebay_{}_products'.format(marketplace.lower())
    today = datetime.date.today()
    self.expire_date = today - datetime.timedelta(days=ttl) 

  def run(self):
    product_ids = []
    batch_size = 500
    try:
      for prod in self.data_source.get_ebay_products(self.marketplace):
        product_ids.append(prod['source_product_id'])
        if len(product_ids) < batch_size:
          continue

        self.process_products(product_ids)
        product_ids = []
    except (KeyboardInterrupt, SystemExit):
      print('Exiting, please wait...')
    finally:
      if product_ids:
        self.process_products(product_ids)
        product_ids = []

  def process_products(self, product_ids):
    pids = dict()
    for pid in product_ids:
      pids[pid] = 'https://www.ebay.com/itm/{}'.format(pid)

    products = False
    max_retries = 3
    while max_retries > 0:
      try:
        products = self.product_service.search_products(self.indice_name, product_ids)
        if not isinstance(products, dict):
          logger.info('[ProductService] Temporary unavailable! Wait for 10 seconds to retry.')
          max_retries -= 1
          time.sleep(10)
          continue

        break
      except Exception as e:
        logger.exception(e)
        max_retries -= 1
        time.sleep(3)

    if not products:
      return

    for pid, product in products.items():
      # existence = product.get('existence', True)
      # if not existence:
      #   continue

      update_date_s = product.get('date', product.get('time', None))
      if not update_date_s:
        continue

      try:
        update_date = dateparser.parse(update_date_s).date()
        # Not expired, no need to update
        if update_date > self.expire_date:
          pids.pop(pid, None)
      except:
        continue

    urls = list(pids.values())
    for product_url in urls:
      if self.last_send_time:
        wait_time = 1 / self.qps - (time.time() - self.last_send_time)
        if wait_time > 0:
          print("Waiting %.3fs to send next message" % wait_time)
          time.sleep(wait_time)

      self.last_send_time = time.time()

      self.r.lpush(self.start_urls_key, json.dumps({'url': product_url}))
      logger.debug('Added %s', product_url)


if __name__ == "__main__":
  feed_seeds()
